"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cli_ux_1 = require("cli-ux");
const chalk_1 = require("chalk");
const abap_1 = require("./lib/abap");
const abap_adt_api_1 = require("abap-adt-api");
const tick = chalk_1.green("\u2713");
class Main {
    // eslint-disable-next-line no-useless-constructor
    constructor(client) {
        this.client = client;
        this.status = {
            processedIncludes: 0,
            processedObjects: 0
        };
    }
    async tryLock(url) {
        try {
            return await this.client.lock(url);
        }
        catch (error) {
            if ((error === null || error === void 0 ? void 0 : error.type) === "ExceptionResourceNoAccess" &&
                !(error === null || error === void 0 ? void 0 : error.message.match(/locked/i)))
                return;
            throw error;
        }
    }
    validateTransport(lock, key, transport) {
        if (lock.IS_LOCAL && transport)
            throw new Error(`Object ${key} is local, can't use ${transport}`);
        if (!lock.IS_LOCAL && !transport)
            throw new Error(`Object ${key} requires a transport ${lock.CORRNR ? `(locked in ${lock.CORRNR})` : ""}`);
        if (transport !== lock.CORRNR && !lock.IS_LOCAL && lock.CORRNR)
            throw new Error(`Object ${key} locked in transport ${lock.CORRNR} can't use ${transport}`);
    }
    async activate(incl) {
        if (incl.type === "PROG/I") {
            const main = await this.client.statelessClone.mainPrograms(incl.metaUrl);
            return this.client.activate(incl.name, incl.sourceUrl, main === null || main === void 0 ? void 0 : main[0]["adtcore:uri"]);
        }
        const active = await this.client.activate(incl.name, incl.sourceUrl);
        if (active.inactive.length > 0) {
            const inactives = abap_adt_api_1.inactiveObjectsInResults(active);
            return this.client.activate(inactives);
        }
        return active;
    }
    async write(incl, formatted, lock, { test, transport }) {
        var _a;
        const key = `${incl.type} ${incl.name}`;
        if (!lock.LOCK_HANDLE)
            throw new Error(`Failed to lock ${key}`);
        this.validateTransport(lock, key, transport);
        cli_ux_1.cli.action.start(`\t${key}`, ` Writing...`);
        if (!test)
            await this.client.setObjectSource(incl.sourceUrl, formatted, lock.LOCK_HANDLE, transport);
        cli_ux_1.cli.action.start(`\t${key}`, ` Unlocking...`);
        await this.client.unLock(incl.sourceUrl, lock.LOCK_HANDLE);
        cli_ux_1.cli.action.start(`\t${key}`, ` Activating...`);
        if (!test) {
            const active = await this.activate(incl);
            if (active.success === false)
                throw new Error(((_a = active.messages[0]) === null || _a === void 0 ? void 0 : _a.shortText) || `Failed to activate ${key}`);
        }
        cli_ux_1.cli.action.start(`\t${tick}${key}`);
    }
    async processInclude(incl, options) {
        this.status.currentInclude = incl;
        const key = `${incl.type} ${incl.name}`;
        cli_ux_1.cli.action.start(`\t${key}`, ` Reading...`);
        const source = await this.client.statelessClone.getObjectSource(incl.sourceUrl);
        cli_ux_1.cli.action.start(`\t${key}`, ` Formatting...`);
        const formatted = await this.client.statelessClone.prettyPrinter(source);
        if (formatted === source)
            cli_ux_1.cli.action.start(`\t${tick}${key} (unchanged)`);
        else {
            cli_ux_1.cli.action.start(`\t${key}`, ` Locking...`);
            const lock = await this.tryLock(incl.sourceUrl);
            if (lock) {
                await this.write(incl, formatted, lock, options);
            }
            else
                cli_ux_1.cli.action.start(`\t${tick}${key} is generated, skipped`);
        }
        cli_ux_1.cli.action.stop();
        this.status.processedIncludes++;
        this.status.currentInclude = undefined;
    }
    async processObjects(objects, options) {
        this.client.stateful = abap_adt_api_1.session_types.stateful;
        try {
            for (const o of objects) {
                this.status.currentObject = o;
                cli_ux_1.cli.action.start(`${o.type} ${o.name}`);
                const includes = await abap_1.expand(this.client, o);
                cli_ux_1.cli.action.start(`${tick}${o.type} ${o.name}`);
                cli_ux_1.cli.action.stop();
                for (const incl of includes)
                    await this.processInclude(incl, options);
                this.status.processedObjects++;
            }
        }
        catch (error) {
            this.client.stateful = abap_adt_api_1.session_types.stateless;
            await this.client.dropSession();
            const { currentInclude, currentObject } = this.status;
            const ref = currentInclude
                ? `include ${currentInclude.type} ${currentInclude.name}`
                : `object ${currentObject === null || currentObject === void 0 ? void 0 : currentObject.type} ${currentObject === null || currentObject === void 0 ? void 0 : currentObject.name}`;
            cli_ux_1.cli.action.start(`Error processing ${ref}`);
            throw error;
        }
    }
}
exports.Main = Main;
